package com.streambase.contrib.EDGAR;


import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import org.slf4j.Logger;
import com.streambase.sb.*;
import com.streambase.sb.adapter.*;
import com.streambase.sb.operator.*;
import com.streambase.sb.util.Msg;
import com.sun.syndication.feed.synd.SyndContent;
import com.sun.syndication.feed.synd.SyndEntry;
import com.sun.syndication.feed.synd.SyndFeed;
import com.sun.syndication.io.FeedException;
import com.sun.syndication.io.SyndFeedInput;
import com.sun.syndication.io.XmlReader;

/**
 * Generated by JDT StreamBase Client Templates (Version: 6.6.10.1012071816).
 *
 * Input adapters take information from outside of an application and
 * provide it in the form of a stream or streams to the application. 
 * Any implementation of this class <b>must</b> extend class InputAdapter.
 * Input adapters should call sendOutput(int, Tuple) to provide tuples 
 * to the application.
 * <p>
 * For in-depth information on implementing a custom adapter, please
 * see "Developing StreamBase Embedded Adapters" in the StreamBase documentation.
 * <p>
 */
public class EDGARMostRecent extends InputAdapter implements Parameterizable,
		Runnable {

	public static final long serialVersionUID = 1294269322713L;
	
	// Properties
	private int pollFrequency;
	private Logger logger;
	// Properties on the GUI
	private java.util.Map<String, String> rssFeeds;
	private String MOST_RECENT = "http://www.sec.gov/cgi-bin/browse-edgar?action=getcurrent&type=&company=&dateb=&owner=include&start=0&count=40&output=atom";
	private static String ADAPTER_SHORT_NAME = "EDGAR Newest Filings";
    private static String ADAPTER_NAME = "SEC EDGAR Newest Filings Input Adapter";    
    
    /** Cached fields */
    private Schema.Field _fldTitle;
    private Schema.Field _fldLink;
    private Schema.Field _fldDescr;
    private Schema.Field _fldCat;
    private Schema.Field _fldUri;
    
    private final String FLD_TITLE = "title";
    private final String FLD_LINK = "link";
    private final String FLD_DESC = "description";
    private final String FLD_CAT = "categories";
    private final String FLD_URI = "uri";
	
    // Various default parameters
    private int PERIOD = 5; // 5-second feed polling interval
	private int SLEEP_TIME = 30000; // Sleep time before re-polling 

    /** Parses RSS or Atom to instantiate a SyndFeed. */
    private SyndFeedInput _input;
    
    /** For each feed, holds the most current times for each item */
    private ConcurrentMap<String, ConcurrentMap<String, Date>> _currentPubDates;
    
    /** Holds the list of feeds for which we've warned about missing fields */
    private Set<String> _alreadyWarnedForMissingFields = new TreeSet<String>();
    
    /** Holds the list of feeds for which we've warned about missing pubDate tags */
    private Set<String> _alreadyWarnedForMissingPubDate = new TreeSet<String>();
    
    private String adapterPrefix = null;
    private static boolean isDebugEnabled = Msg.debug();
	public static int count = 0;

	/**
	 * The constructor is called when the Adapter instance is created, but before the Adapter 
	 * is connected to the StreamBase application. We recommended that you set the initial input 
	 * port and output port count in the constructor by calling setPortHints(inPortCount, outPortCount). 
	 * The default is no input ports or output ports. The constructor may also set default values for 
	 * adapter parameters. These values will be displayed in StreamBase Studio when a new adapter is 
	 * dragged to the canvas, and serve as the default values for omitted optional parameters.
	 */
	public EDGARMostRecent() {
		super();		
		logger = getLogger();
		setPortHints(0, 1);
		setDisplayName(ADAPTER_NAME);
		setShortDisplayName(ADAPTER_SHORT_NAME);
		setPollFrequency(PERIOD);
		rssFeeds = new java.util.LinkedHashMap<String, String>();
		_input = new SyndFeedInput();
	}

	/**
	 * Typecheck this adapter. The typecheck method is called after the adapter instance is
	 * connected in the StreamBase application. The adapter should validate its parameters 
	 * and throw TypecheckException if any problems are found. The message associated with 
	 * the thrown TypecheckException will be displayed in StreamBase Studio during authoring,
	 * or printed on the console by sbd. Input adapters should set the schema of each output
	 * port by calling the setOutputSchema(portNum, schema) method for each output port. 
	 * If the adapter needs to change the number of input ports based on parameter values,
	 * it should call requireInputPortCount(portCount) at this point. 
	 */
	public void typecheck() throws TypecheckException {
		// Go over what rss fields the user wants, creating a schema from it
        List<Schema.Field> fields = new ArrayList<Schema.Field>();
        // set size of the fields list/schema        
        fields.add(Schema.createField(DataType.STRING, FLD_TITLE));
        fields.add(Schema.createField(DataType.STRING, FLD_LINK));
        fields.add(Schema.createField(DataType.STRING, FLD_DESC));
        fields.add(Schema.createField(DataType.STRING, FLD_CAT));
        fields.add(Schema.createField(DataType.STRING, FLD_URI));       
     	
        Schema output0 = new Schema(null, fields);
        setOutputSchema(0, output0);        
        
        if (getPollFrequency() <= 0)
            throw new PropertyTypecheckException(EDGARReaderBeanInfo.PERIOD_PROPERTY_NAME, "Polling Frequency property must be positive");
	}

	/**
	 * Initialize the adapter. If typecheck succeeds, the init method is called before
	 * the StreamBase application is started. Note that your adapter is not required to
	 * define the init method, unless you need to register a runnable or perform
	 * initialization of a resource such as, for example, a JDBC pool.
	 */
	public void init() throws StreamBaseException {
		super.init();		
		// Put the actual RSS feeds into the current current published dates map				
		rssFeeds.put("MostRecent", MOST_RECENT);
        _currentPubDates = new ConcurrentHashMap<String, ConcurrentMap<String,Date>>(1);
        _currentPubDates.put("MostRecent", new ConcurrentHashMap<String, Date>());
		// Register the object so it will be run as a thread managed by StreamBase.
		registerRunnable(this, true);	
	}

	/**
	 * Shutdown adapter by cleaning up any resources used (e.g. close an output file
	 * if it has been opened). When the application is shutting down, the adapter's shutdown.
	 * method will be called first. Once this has returned, all threads should exit and the
	 * adapter is considered shutdown.
	 */
	public void shutdown() {

	}

	/**
	 * Main thread of the adapter. At this point, the application begins to run.
	 * StreamBase will start threads for any managed runnables registered by
	 * earlier calls to registerRunnable. Input adapters can call sendOutput(port, tuple) 
	 * at any time to output a tuple to the specified port.
	 */
	public void run() {
		// fill the tuple and output the tuple
		fillAndOutputTuple();
	}


    protected void fillAndOutputTuple() {    	
        try {
            //
            // Cache tuple and fields
            //
            Schema schema = getOutputSchema(0);
            Tuple tuple = schema.createTuple();
            try
            {
                _fldTitle = schema.getField(FLD_TITLE);
                _fldLink = schema.getField(FLD_LINK);
                _fldDescr = schema.getField(FLD_DESC);
                _fldCat = schema.getField(FLD_CAT);
                _fldUri = schema.getField(FLD_URI);
            }
            catch (TupleException e1)
            {
                error("Error caching tuple fields: " + e1.toString() + "\n");
            }

            while (shouldRun()) {            	
            	Set<String> feedNames = rssFeeds.keySet();             	
                for (String name : feedNames){
                    //
                    // Load the feed, regardless of RSS or Atom type
                    //
                    SyndFeed feed;
                    ConcurrentMap<String, Date> dateCache = _currentPubDates.get(name);
                    try {
                        String sFeed = rssFeeds.get(name);
                        URL rssLink = new URL(sFeed);
                        feed = null;

                        //
                        // Try to access the feed NumRetries times (forever if it's -1),
                        // in case there's a timeout.
                        //
                        int nRetries = -1;
                        while (nRetries != 0)
                        {
                            try
                            {
                                feed = _input.build(new XmlReader(rssLink));
                                break;
                            }
                            catch (java.io.IOException ioe)
                            {
                                warn(String.format("%s: Could not connect to feed '%s': %s",
                                                       this.getName(), sFeed, ioe.toString()));
                                
                                //
                                // Sleep some before trying again
                                //
                                try
                                {
                                    Thread.sleep(SLEEP_TIME); // Wait 30 seconds between retries
                                }
                                catch (InterruptedException e1)
                                {
                                }
                                if (nRetries != -1)
                                    nRetries--;
                            }
                        }
                                            
                        List<?> entries = feed.getEntries();
                        if (entries.size() == 0)
                            continue;       // No entries in this feed!
                        
                        //
                        // Ok, we have a list of all the entries in this feed.
                        // Next we have to determine which of these to include
                        // in our update to the SB app.
                        //
                        // This will be done by looking at the (optional!) various publishing date
                        // entries of each item, and check it against the previous
                        // pub date for this item. If the pub date is newer, we add it
                        // to our itemsToSend list.
                        //
                        List<SyndEntry> itemsToSend = new ArrayList<SyndEntry>();

                        for (Object obj : entries)
                        {
                            //
                            // Grab the title and the date of last update for this entry.
                            //
                            // (Note that it is possible that this entry has no date associated with it,
                            // since all RSS date tags are optional...)
                            //
                            SyndEntry se = (SyndEntry)obj;
                            
                            String sCurrentEntryTitle = se.getTitle();
                            Date newDate = se.getUpdatedDate();
                            if (newDate == null)
                                newDate = se.getPublishedDate();
                            if (newDate == null)
                                newDate = feed.getPublishedDate();
                            if (newDate == null)
                            {
                                newDate = new Date(0L); // A basically invalid date, signifying that none was specified.
                                if (_alreadyWarnedForMissingPubDate.add(sFeed))
                                {
                                    warn(String.format("%s: Some or all entries of feed '%s' will not be updated because no publish date was found for them.",
                                                           this.getName(), sFeed));
                                }
                            }

                            //
                            // First check if we have cached this entry.
                            // If we haven't, it means it's a new entry so
                            // we need to cache it now and send an update for it.
                            //
                            if (!dateCache.containsKey(sCurrentEntryTitle))
                            {
                                dateCache.put(sCurrentEntryTitle, newDate);
                                itemsToSend.add(se);
                                if (isDebugEnabled) debug(String.format("%s: Adding '%s' to date cache of feed '%s'\n", this.getName(), sCurrentEntryTitle, sFeed));
                                continue;
                            }
                            
                            //
                            // Ok, so we've seen this entry before. Now let's check if
                            // this is a newer version -- in which case we need to send an update.
                            //
                            Date previousDate = dateCache.get(sCurrentEntryTitle);
                            
                            if (newDate.after(previousDate))
                            {
                                //
                                // This entry has changed since we last saw it, so update
                                // our cache entry and send an update.
                                //
                                dateCache.put(sCurrentEntryTitle, newDate);
                                itemsToSend.add(se);
                                if (isDebugEnabled) debug(String.format("%s: Updating '%s' in date cache of feed '%s'\n", this.getName(), sCurrentEntryTitle, sFeed));
                            }
                        }

                        //
                        // Housekeeping: If there are entries in our cache that are no longer
                        // part of the feed, we need to remove them.
                        //
                        for (String sTitle : dateCache.keySet())
                        {
                            boolean bFound = false;
                            for (Object obj : entries)
                            {
                                SyndEntry se = (SyndEntry)obj;
                                if (se.getTitle().equals(sTitle))
                                {
                                    bFound = true;
                                    break;
                                }
                            }
                            if (!bFound)
                            {
                                if (isDebugEnabled) debug(String.format("%s: Removing '%s' from date cache of feed '%s'\n", this.getName(), sTitle, sFeed));
                                dateCache.remove(sTitle);
                            }
                        }

                        //
                        // Fill out tuple
                        //
                        List<String> lstFieldsToWarn = null;
                        boolean bWarn = !_alreadyWarnedForMissingFields.contains(sFeed);
                        if (bWarn)
                            lstFieldsToWarn = new ArrayList<String>();

                        for (SyndEntry e : itemsToSend) {
                            tuple.clear();                            
                            String s = e.getTitle();
                            if (s != null)
                            {
                                tuple.setString(_fldTitle, s);
                            }
                            else if (bWarn)
                                lstFieldsToWarn.add(_fldTitle.getName());

                            s = e.getLink();
                            if (s != null)
                            {
                                tuple.setString(_fldLink, s);
                            }
                            else if (bWarn)
                                lstFieldsToWarn.add(_fldLink.getName());

                            SyndContent descr = e.getDescription();
                            if (descr != null)
                            {
                                s = descr.getValue();
                                tuple.setString(_fldDescr, s);
                            }
                            else if (bWarn)
                                lstFieldsToWarn.add(_fldDescr.getName());

                            List<?> categories = e.getCategories();
                            if (categories != null && categories.size() > 0)
                            {
                                StringBuilder sb = new StringBuilder();
                                for (int x = 0; x < categories.size(); x++)
                                {
                                    if (x != 0)
                                        sb.append(",");
                                    sb.append(categories.get(x).toString());
                                }
                                s = sb.toString();
                                tuple.setString(_fldCat, s);
                            }
                            else if (bWarn)
                                lstFieldsToWarn.add(_fldCat.getName());
                            
                     
                            if (e.getUri() != null)
                            {
                                s = e.getUri();
                                tuple.setString(_fldUri, s);
                            }
                            else if (bWarn)
                                lstFieldsToWarn.add(_fldUri.getName());
                            
                            
                            //
                            // Warn if some fields could not be set
                            //
                            if (bWarn && lstFieldsToWarn.size() > 0)
                            {
                                StringBuilder sbMsg = new StringBuilder("The following fields will be set to null for feed '");
                                sbMsg.append(sFeed);
                                sbMsg.append("' because they were unavailable:\n");
                                for (String sField : lstFieldsToWarn)
                                    sbMsg.append(String.format("\t\t%s\n", sField));
                                sbMsg.append("       (The fields may not be part of the specification for this feed's version, or they may simply be optional and were not specified.)");
                                warn(sbMsg.toString());
                                _alreadyWarnedForMissingFields.add(sFeed);
                                bWarn = false;
                            }

                            //
                            // Send update
                            //
                            try {
                                if (isDebugEnabled) debug(String.format("%s: Sending update tuple for feed '%s':\n\t%s\n", this.getName(), sFeed, tuple.toString()));
                                sendOutput(0, tuple);
                            } catch (StreamBaseException se) {
                                logError(getAdapterPrefix()+"Exception sending output to port 0");
                                System.err.println(se);
                                return;
                            }
                        }
                    } catch (StreamBaseException sbe) {
                        //Exception already logged.
                        return;
                    } catch (FeedException e) {
                    	StringWriter sw = new StringWriter();
                    	e.printStackTrace(new PrintWriter(sw));
                        logError(getAdapterPrefix()+"Error in feed: " + name + "; " + sw);
                    } catch (IOException e) {
                        logError(e.toString());
                        return;
                    }
                }
                
                try {
                    Thread.sleep(getPollFrequency()*1000);
                } catch (InterruptedException e) {
                }
            }
        } finally {
            info("Exiting enqueuing thread");
            try {
                //nothing to close
            } catch (Exception e) {
                logError(e.toString());
            }
        }
    }
    

    
    @Override
    public URL getIconResource(IconKind iconType) {
        switch (iconType) {
           case CANVAS_OVERLAY_ICON:
               return getClass().getResource("rss_reader_24x24.png");
           case PALETTE_ICON_LARGE:
               return getClass().getResource("rss_reader_32x32.png");
           case PALETTE_ICON_SMALL:
               return getClass().getResource("rss_reader_16x16.png");
       }
       return null;
    }

	/***************************************************************************************
	 *																					   *
	 * Error Messages and Debugging Section          								       *
	 * 																					   *
	 ***************************************************************************************/
    
    protected void logError(String message)
    {
    	logger.error(message);
    }
    
    /**
     * method to add the adapter name and user-supplied name of the icon.
     * @return string
     */
    public String getAdapterPrefix() {
        if (adapterPrefix == null) { 
            String instance = getName();
            adapterPrefix = ADAPTER_SHORT_NAME + (instance == null ? "" :  "(" + instance + ")") + ": ";
        } 
        return adapterPrefix; 
    }
    /**
     * General method for the error messages 
     * @param message
     */  
    public void error(String message) {
        Msg.error(getAdapterPrefix() + message);
    }
    /**
     * General method for the warn messages 
     * @param message
     */ 
    public void warn(String message) {
        Msg.warn(getAdapterPrefix() + message);
    }
    /**
     * General method for the debug messages 
     * @param message
     */  
      public void debug(String message) {
          Msg.debug(getAdapterPrefix() + message);
      }
      /**
       * General method for the info messages 
       * @param message
       */ 
      public void info(String message) {
          Msg.info(getAdapterPrefix() + message);
      } 
      public boolean debug()
      {
          return Msg.debug();
      }
    
	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	public void setPollFrequency(int pollFrequency) {
		this.pollFrequency = pollFrequency;
	}

	public int getPollFrequency() {
		return this.pollFrequency;
	}

}
