package com.streambase.contrib.EDGAR;

import java.beans.*;

import com.streambase.sb.operator.parameter.*;

/**
 * A BeanInfo class controls what properties are exposed, add 
 * metadata about properties (such as which properties are optional), and access 
 * special types of properties that can't be automatically derived via reflection. 
 * If a BeanInfo class is present, only the properties explicitly declared in
 * this class will be exposed by StreamBase.
 */
public class EDGAROnlineBeanInfo extends SimpleBeanInfo {
	public static final String PERIOD_PROPERTY_NAME = "Period";
	public static final String AGG_PROPERTY_NAME = "Aggregate"; 
	public static final String INSIDER_PROPERTY_NAME = "NonInsider";
	public static final String ALL_PROPERTY_NAME = "AllFeeds";
	public static final String NO_PROPERTY_NAME = "NoFeeds";

	/*
	 * The order of properties below determines the order they are displayed within
	 * the StreamBase Studio property view. 
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		try {
			PropertyDescriptor[] p = {
					new SBPropertyDescriptor("pollFrequency", EDGAROnline.class)
							.displayName("pollFrequency").description("The time, in seconds, to wait between looking for new feed entries."),
					new SBPropertyDescriptor("allFilings", EDGAROnline.class)
							.displayName("All Filings").description("Subscribe to this channel to keep on top of the latest significant SEC filings, including all insider transactions.")
							.optional(),
					new SBPropertyDescriptor("nonInsiderFilings",
							EDGAROnline.class).displayName(
							"Non-Insider Filings").description("Subscribe to this channel to keep on top of the latest significant SEC filings, excluding all insider transactions.").optional(),
					new SBPropertyDescriptor("ipoFilings", EDGAROnline.class)
							.displayName("IPO Filings").description("Subscribe to this channel to keep on top of the latest Initial Public Offering related filings.")
							.optional(),
					new SBPropertyDescriptor("annualReports", EDGAROnline.class)
							.displayName("Annual Reports").description("Subscribe to this channel to keep on top of the latest Initial Public Offering related filings.")
							.optional(),
					new SBPropertyDescriptor("quarterlyReports",
							EDGAROnline.class).displayName("Quarterly Reports")
							.description("Subscribe to this channel to keep on top of the latest Annual Reports (10-Q).").optional(),
					new SBPropertyDescriptor("proxyStatements",
							EDGAROnline.class).displayName("Proxy Statements")
							.description("Subscribe to this channel to keep on top of the latest Proxy Statements (DEF/PRE).").optional(),
					new SBPropertyDescriptor("eventFilings", EDGAROnline.class)
							.displayName("Current Event Filings").description("Subscribe to this channel to keep on top of the latest Current Event Filings (8-K).").optional(),
					new SBPropertyDescriptor("mutualFund", EDGAROnline.class)
							.displayName("Mutual Fund Filings").description("Subscribe to this channel to keep on top of the latest Mutual Fund Filings.")
							.optional()};
			return p;
		} catch (IntrospectionException e) {
			System.err.println("Failed to create property descriptors");
			e.printStackTrace();
			return null;
		}
	}

}
