package com.streambase.contrib.EDGAR;

import java.beans.*;

import com.streambase.sb.operator.parameter.*;

/**
 * A BeanInfo class controls what properties are exposed, add 
 * metadata about properties (such as which properties are optional), and access 
 * special types of properties that can't be automatically derived via reflection. 
 * If a BeanInfo class is present, only the properties explicitly declared in
 * this class will be exposed by StreamBase.
 */
public class EDGARReaderBeanInfo extends SimpleBeanInfo {
    public static final String RSS_FEEDS_PROPERTY_NAME = "RSSFeeds";
    public static final String PERIOD_PROPERTY_NAME = "Period";
    public static final String MAX_STRING_SIZE_PROPERTY_NAME = "maxStringSize";
    public static final String NUM_RETRIES_PROPERTY_NAME = "NumRetries";
	public static final String CONTROL_PORT = "Start Control Port";
	
	/*
	 * The order of properties below determines the order they are displayed within
	 * the StreamBase Studio property view. 
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		try {
			PropertyDescriptor[] p = {
					new SBPropertyDescriptor("rssFeeds", EDGARReader.class)
							.displayName("RSS Feeds").description("List of your Key Values and CIK's (dynamic mode) or RSS/Atom feeds (static mode)").optional(),
					new SBPropertyDescriptor("pollFrequency",
							EDGARReader.class).displayName(
							"Poll Frequency").description("The time, in seconds, to wait between looking for new feed entries"),
					new SBPropertyDescriptor("connectionRetries",
							EDGARReader.class).displayName(
							"Number of Connection Retries").description("Number of times the adapter should try to reconnect when timing out while accessing a feed (-1 to try forever)")};
				return p;
		} catch (IntrospectionException e) {
			System.err.println("Failed to create property descriptors");
			e.printStackTrace();
			return null;
		}
	}

}
