package com.streambase.contrib.EDGAR;

import com.streambase.sb.Schema;
import com.streambase.sb.operator.TypecheckException;

public class SchemaValidator {
	
	/**
	 * This function tests to see whether the appropriate fields of a input schema are present
	 * @param testSchema the incoming input schema
	 * @param referenceSchema the schema with which to test fields against
	 * @throws TypecheckException
	 */
	public static void validateSchema(Schema testSchema, Schema referenceSchema) throws TypecheckException {
		for (Schema.Field curField : referenceSchema.fields()) {
			try {												
				Schema.Field testField = testSchema.getField(curField.getName());				
				// Field is present but is of the wrong type
				if (testField.getDataType() != curField.getDataType())
				{
					throw new TypecheckException("Field '"
						+ curField.getName() + "' has wrong type.  Should be "
						+ curField.getDataType().getName() + ", but is "
						+ testField.getDataType().getName() + ".");
				}
			}
			catch (Exception exc) {
				// Field is not present 
				throw new TypecheckException("Schema is missing required field '" 
						+ curField.getName() + "', type " 
						+ curField.getDataType().getName());
			}
		}
	}
}
